@@ -11,11 +11,11 @@ import zlib
 import bisect
 from hashlib import md5, sha1
 
-from ._compat import xrange, b
+from ._compat import xrange, b, long
 
 
 hash_methods = {
-    'crc32': zlib.crc32,
+    'crc32': lambda x: zlib.crc32(x) & 0xffffffff,
     'md5': lambda x: long(md5(x).hexdigest(), 16),
     'sha1': lambda x: long(sha1(x).hexdigest(), 16),
 }
@@ -47,9 +47,9 @@ class HashRing(object):
         """
         self.nodes.append(node)
         for x in xrange(self.replicas):
-            ringkey = self.hash_method(b("%s:%d" % (node, x)))
-            self.ring[ringkey] = node
-            self.sorted_keys.append(ringkey)
+            ring_key = self.hash_method(b("%s:%d" % (node, x)))
+            self.ring[ring_key] = node
+            self.sorted_keys.append(ring_key)
 
         self.sorted_keys.sort()
 
@@ -58,9 +58,9 @@ class HashRing(object):
         """
         self.nodes.remove(node)
         for x in xrange(self.replicas):
-            ringkey = self.hash_method(b("%s:%d" % (node, x)))
-            self.ring.remove(ringkey)
-            self.sorted_keys.remove(ringkey)
+            ring_key = self.hash_method(b("%s:%d" % (node, x)))
+            self.ring.pop(ring_key)
+            self.sorted_keys.remove(ring_key)
 
     def get_node(self, key):
         """Given a string key a corresponding node in the hash ring is returned.
@@ -80,8 +80,8 @@ class HashRing(object):
             return [None, None]
         crc = self.hash_method(b(key))
         idx = bisect.bisect(self.sorted_keys, crc)
+        # prevents out of range index
         idx = min(idx, (self.replicas * len(self.nodes)) - 1)
-                  # prevents out of range index
         return [self.ring[self.sorted_keys[idx]], idx]
 
     def iter_nodes(self, key):
