@@ -7,7 +7,9 @@ except ImportError:
     # python3
     from urllib.parse import unquote
     from urllib.parse import urlparse
-    from cgi import parse_qsl  # noqa
+    from urllib.parse import parse_qsl
+
+from redis.connection import URL_QUERY_ARGUMENT_PARSERS
 
 
 def _parse_url(url):
@@ -35,4 +37,11 @@ def _parse_url(url):
 def parse_url(url):
     scheme, host, port, user, password, db, query = _parse_url(url)
     assert scheme == 'redis'
+    for param, value in query.items():
+        if param in URL_QUERY_ARGUMENT_PARSERS:
+            parser = URL_QUERY_ARGUMENT_PARSERS[param]
+            query[param] = parser(value)
     return dict(host=host, port=port, password=password, db=db, **query)
+
+if __name__ == '__main__':
+    print(parse_url('redis://127.0.0.1:6379/1?name=node2&max_connections=100'))
